/*
 * Example for TouchKey 2 click
 *
 * Date         Feb 2017
 * Author       Stefan Popovic
 * Copyright    GNU Public License v2
 *
* Test configuration PIC :
 *  MCU           :	P18F87K22
 *  Dev. Board    :	EasyPIC PRO v7
 *  SW            :	mikroC PRO for PIC v7.0.0
 *
 *  NOTE          : - Place TouchKey 2 click board on mikroBUS slot 1
 *                  - Connect easyTFT and turn backlight ON ( turn SW4.2 )
 *                  - This click board sends 2 bytes via UART, on key pressed it
 *                    sends KEY_Byte and on key release it sends 0x00
 *
/*******************************************************************************
* Includes
*******************************************************************************/
#include "tft_resources.h"
#include <stdbool.h>
#include <stdint.h>

/*******************************************************************************
* Module Preprocessor Constants
*******************************************************************************/
#define KEY_A                0x01
#define KEY_B                0x02
#define KEY_C                0x04
#define KEY_D                0x08
#define KEY_RELEASE          0x00

/*******************************************************************************
* Module Variable Definitions
*******************************************************************************/
sbit TOUCH_KEY_2_RST      at RC0_bit;
sbit TOUCH_KEY_2_RST_DIR  at TRISC0_bit;
sbit TOUCH_KEY_2_CS       at RE0_bit;
sbit TOUCH_KEY_2_CS_DIR   at TRISE0_bit;

bool key_A_pressed  = false;
bool key_B_pressed  = false;
bool key_C_pressed  = false;
bool key_D_pressed  = false;
bool key_released   = false;

char key;
char old_key;

/*******************************************************************************
* Function Declarations
*******************************************************************************/
static void system_init( void );
static void check_key_event( void );
static void display_key( char* key_name, unsigned int color );

/*******************************************************************************
* Function Definitions
*******************************************************************************/

/*
 * System Initialization
 *
 * Configuring MCU ports, initializing UART, enabling interrupt
 * and performing reset
 */
static void system_init( void )
{
    // Configuring RST and CS pins as output
    TOUCH_KEY_2_RST_DIR = 0;
    TOUCH_KEY_2_CS_DIR  = 0;

    // UART Initialization
    UART1_Init( 9600 );
    Delay_ms( 100 );

    // UART Interrupt enabling
    RC1IE_bit = 1;
    RC1IF_bit = 0;
    PEIE_bit  = 1;
    GIE_bit   = 1;

    // Perform TouchKey 2 board reset
    TOUCH_KEY_2_RST = 1;
    Delay_ms( 100 );
    TOUCH_KEY_2_RST = 0;
    Delay_ms( 100 );
    TOUCH_KEY_2_RST = 1;
}

/*
 * Checking for key pressed events
 *
 * Looking for Key ID and setting appropriate flags. Called inside interrupt
 */
static void check_key_event ( void )
{
        if ( key == KEY_A )
             {
                key_A_pressed = true;
             }
        else if ( key == KEY_B )
             {
                key_B_pressed = true;
             }
         else if ( key == KEY_C )
             {
                key_C_pressed = true;
             }
         else if ( key == KEY_D )
             {
                key_D_pressed = true;
             }
         else if ( key == KEY_RELEASE )
             {
                key_released = true;
             }
}

/*
 * Displaying key name with color
 */
void display_key( unsigned char* key_name, unsigned int color )
{
  TFT_Set_Font( &LilyUPC57x120_Bold, color, FO_HORIZONTAL );
  TFT_Rectangle( 50, 80, 300, 165 );
  TFT_Write_Text( key_name, 135, 80 );
  TFT_Set_Font( &HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL );
  TFT_Write_Text( "Key", 55, 120 );
  TFT_Write_Text( "Pressed.", 205, 120 );
}

/*
 * Main function
 */
void main( void )
{
    system_init();
    display_init();

    TFT_Set_Font( &HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL );
    TFT_Write_Text( "Press Any On-board Key", 50, 120 );

    while( true ) 
    {
        // Key A has been pressed
        if( key_A_pressed && old_key != KEY_A )
        {   
            key_A_pressed = false;
            old_key = KEY_A;
            display_key( "A", CL_RED );
        }
        // Key B has been pressed
        else if( key_B_pressed && old_key != KEY_B )
        {
            key_B_pressed = false;
            old_key = KEY_B;
            display_key( "B", CL_GREEN );
        }
        // Key C has been pressed
        else if( key_C_pressed && old_key != KEY_C )
        {
            key_C_pressed = false;
            old_key = KEY_C;
            display_key( "C", CL_BLUE );
        }
        // Key D has been pressed
        else if( key_D_pressed && old_key != KEY_D )
        {
            key_D_pressed = false;
            old_key = KEY_D;
            display_key( "D", CL_PURPLE );
        }
        // Key released
        else if ( key_released && old_key != KEY_RELEASE )
        {
           key_released = false;
           TFT_Rectangle( 50, 80, 300, 165 );
           TFT_Set_Font( &HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL );
           TFT_Write_Text( "Key", 55, 120 );
           TFT_Write_Text( "Released.", 205, 120 );
           TFT_Set_Font( &LilyUPC57x120_Bold, CL_GRAY, FO_HORIZONTAL );
           if ( old_key == KEY_A )
           {
              TFT_Write_Text( "A", 135, 80 );
           }
           else if ( old_key == KEY_B )
           {
              TFT_Write_Text( "B", 135, 80 );
           }
           else if ( old_key == KEY_C )
           {
              TFT_Write_Text( "C", 135, 80 );
           }
           else if ( old_key == KEY_D )
           {
              TFT_Write_Text( "D", 135, 80 );
           }

           old_key = KEY_RELEASE;
        }
    }
}

void interrupt()
{
   if( RC1IF_bit == 1 )
   {
     char tmp = UART1_Read();
     key = tmp;
     check_key_event();
   }
}
/************************************************************ END OF FILE *****/